@@ -22,10 +22,10 @@ static void lookatdir(const char *dname)
 	HXdir_close(dh);
 }
 
-int main(int argc, const char **argv)
+int main(int argc, char **argv)
 {
 	if (HX_init() <= 0)
-		abort();
+		return EXIT_FAILURE;
 	if (argc == 1) {
 		/* On Windows VCRT, "/" yields nothing, "c:/" is needed */
 		lookatdir("/");
