@@ -1,10 +1,6 @@
+// SPDX-License-Identifier: MIT
 /*
  *	Behavior Correctness Test for HX_strchr2
- *	Copyright Jan Engelhardt, 2013
- *
- *	This program is free software; you can redistribute it and/or
- *	modify it under the terms of the WTF Public License version 2 or
- *	(at your option) any later version.
  */
 #include <libHX/string.h>
 
